@@ -60,6 +60,7 @@ typedef struct {
 
 	GtkWidget *minimum_size_spin;
 	GtkWidget *maximum_size_spin;
+	GtkWidget *about;
 
 	/* gconf listeners id */
 	guint listeners [5];
@@ -93,37 +94,29 @@ tasklist_update (TasklistData *tasklist)
 	wnck_tasklist_set_switch_workspace_on_unminimize (WNCK_TASKLIST (tasklist->tasklist),
 							  tasklist->move_unminimized_windows);
 }
+
 static void
-response_cb(GtkWidget * widget,int id, TasklistData *tasklist)
+response_cb (GtkWidget    *widget,
+	     int           id,
+	     TasklistData *tasklist)
 {
-	if(id == GTK_RESPONSE_HELP) {
-
+	if (id == GTK_RESPONSE_HELP) {
 		GError *error = NULL;
-		static GnomeProgram *applet_program = NULL;
-
-		if (!applet_program) {
-			int argc = 1;
-			char *argv[2] = { "window-list" };
-			applet_program = gnome_program_init ("window-list", VERSION,
-							     LIBGNOME_MODULE,argc, argv,
-							     GNOME_PROGRAM_STANDARD_PROPERTIES,NULL);
-		}
 
 		egg_help_display_desktop_on_screen (
-			applet_program, "window-list",
-			"window-list", "windowlist-prefs",
+			NULL, "window-list", "window-list", "windowlist-prefs",
 			gtk_widget_get_screen (tasklist->applet),
 			&error);
 		if (error) {
 			GtkWidget *dialog;
-			dialog = gtk_message_dialog_new (GTK_WINDOW(widget),
+			dialog = gtk_message_dialog_new (GTK_WINDOW (widget),
 							 GTK_DIALOG_DESTROY_WITH_PARENT,
 							 GTK_MESSAGE_ERROR,
-							 GTK_BUTTONS_CLOSE,
+							 GTK_BUTTONS_OK,
 							  _("There was an error displaying help: %s"),
 							 error->message);
 
-			g_signal_connect (G_OBJECT (dialog), "response",
+			g_signal_connect (dialog, "response",
 					  G_CALLBACK (gtk_widget_destroy),
 					  NULL);
 	
@@ -133,8 +126,7 @@ response_cb(GtkWidget * widget,int id, TasklistData *tasklist)
 			gtk_widget_show (dialog);
 			g_error_free (error);
 		}
-	}
-	else
+	} else
 		gtk_widget_hide (widget);
 }
 
@@ -266,6 +258,9 @@ destroy_tasklist(GtkWidget * widget, TasklistData *tasklist)
 	if (tasklist->properties_dialog)
 		gtk_widget_destroy (tasklist->properties_dialog);
 
+	if (tasklist->about)
+		gtk_widget_destroy (tasklist->about);
+
         g_free (tasklist);
 }
 
@@ -741,19 +736,9 @@ display_help_dialog (BonoboUIComponent *uic,
 		     const gchar       *verbname)
 {
 	GError *error = NULL;
-	static GnomeProgram *applet_program = NULL;
-
-	if (!applet_program) {
-		int argc = 1;
-		char *argv[2] = { "window-list" };
-		applet_program = gnome_program_init ("window-list", VERSION,
-						     LIBGNOME_MODULE,argc, argv,
-						     GNOME_PROGRAM_STANDARD_PROPERTIES,NULL);
-	}
 
 	egg_help_display_desktop_on_screen (
-			applet_program, "window-list",
-			"window-list", NULL,
+			NULL, "window-list", "window-list", NULL,
 			gtk_widget_get_screen (tasklist->applet),
 			&error);
 	if (error) {
@@ -761,11 +746,11 @@ display_help_dialog (BonoboUIComponent *uic,
 		dialog = gtk_message_dialog_new (NULL,
 						 GTK_DIALOG_MODAL,
 						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
+						 GTK_BUTTONS_OK,
 						  _("There was an error displaying help: %s"),
 						 error->message);
 
-		g_signal_connect (G_OBJECT (dialog), "response",
+		g_signal_connect (dialog, "response",
 				  G_CALLBACK (gtk_widget_destroy),
 				  NULL);
 
@@ -782,7 +767,6 @@ display_about_dialog (BonoboUIComponent *uic,
 		      TasklistData      *tasklist,
 		      const gchar       *verbname)
 {
-	static GtkWidget *about = NULL;
 	GdkPixbuf *pixbuf = NULL;
 	gchar *file;
 	
@@ -796,11 +780,11 @@ display_about_dialog (BonoboUIComponent *uic,
 	};
 	const char *translator_credits = _("translator_credits");
 
-	if (about) {
-		gtk_window_set_screen (GTK_WINDOW (about),
+	if (tasklist->about) {
+		gtk_window_set_screen (GTK_WINDOW (tasklist->about),
 				       gtk_widget_get_screen (tasklist->applet));
-		gtk_widget_show (about);
-		gtk_window_present (GTK_WINDOW (about));
+		gtk_widget_show (tasklist->about);
+		gtk_window_present (GTK_WINDOW (tasklist->about));
 		return;
 	}
 
@@ -808,7 +792,7 @@ display_about_dialog (BonoboUIComponent *uic,
 	pixbuf = gdk_pixbuf_new_from_file (file, NULL);
 	g_free(file);
 
-	about = gnome_about_new (_("Window List"), VERSION,
+	tasklist->about = gnome_about_new (_("Window List"), VERSION,
 				 "Copyright \xc2\xa9 2001-2002 Red Hat, Inc.",
 				 _("The Window List shows a list of all visible windows and lets you browse them."),
 				 authors,
@@ -816,19 +800,19 @@ display_about_dialog (BonoboUIComponent *uic,
 				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 				 pixbuf);
 	
-	gtk_window_set_wmclass (GTK_WINDOW (about), "tasklist", "Tasklist");
-	gtk_window_set_screen (GTK_WINDOW (about),
+	gtk_window_set_wmclass (GTK_WINDOW (tasklist->about), "tasklist", "Tasklist");
+	gtk_window_set_screen (GTK_WINDOW (tasklist->about),
 			       gtk_widget_get_screen (tasklist->applet));
 
 	if (pixbuf) {
-		gtk_window_set_icon (GTK_WINDOW (about), pixbuf);
+		gtk_window_set_icon (GTK_WINDOW (tasklist->about), pixbuf);
 		g_object_unref (pixbuf);
 	}
 	
-	g_signal_connect (G_OBJECT(about), "destroy",
-			  (GCallback)gtk_widget_destroyed, &about);
+	g_signal_connect (G_OBJECT(tasklist->about), "destroy",
+			  (GCallback)gtk_widget_destroyed, &tasklist->about);
 	
-	gtk_widget_show (about);
+	gtk_widget_show (tasklist->about);
 }
 
 static void
@@ -900,6 +884,44 @@ spin_maximum_size_changed (GtkSpinButton *button, TasklistData *tasklist)
 	panel_applet_gconf_set_int (applet, "maximum_size", prop_value, NULL);
 }
 
+static void
+setup_sensitivity (TasklistData *tasklist,
+		   GConfClient *client,
+		   GladeXML *xml,
+		   const char *wid1,
+		   const char *wid2,
+		   const char *wid3,
+		   const char *key)
+{
+	PanelApplet *applet = PANEL_APPLET (tasklist->applet);
+	char *fullkey;
+	GtkWidget *w;
+
+	fullkey = panel_applet_gconf_get_full_key (applet, key);
+
+	if (gconf_client_key_is_writable (client, fullkey, NULL)) {
+		g_free (fullkey);
+		return;
+	}
+	g_free (fullkey);
+
+	w = glade_xml_get_widget (xml, wid1);
+	g_assert (w != NULL);
+	gtk_widget_set_sensitive (w, FALSE);
+
+	if (wid2 != NULL) {
+		w = glade_xml_get_widget (xml, wid2);
+		g_assert (w != NULL);
+		gtk_widget_set_sensitive (w, FALSE);
+	}
+	if (wid3 != NULL) {
+		w = glade_xml_get_widget (xml, wid3);
+		g_assert (w != NULL);
+		gtk_widget_set_sensitive (w, FALSE);
+	}
+
+}
+
 #define WID(s) glade_xml_get_widget (xml, s)
 
 static void
@@ -915,14 +937,45 @@ setup_dialog (GladeXML     *xml,
 
 	tasklist->show_current_radio = WID ("show_current_radio");
 	tasklist->show_all_radio = WID ("show_all_radio");
+
+	setup_sensitivity (tasklist, client, xml,
+			   "show_current_radio",
+			   "show_all_radio",
+			   NULL,
+			   "display_all_workspaces" /* key */);
+
 	tasklist->never_group_radio = WID ("never_group_radio");
 	tasklist->auto_group_radio = WID ("auto_group_radio");
 	tasklist->always_group_radio = WID ("always_group_radio");
+
+	setup_sensitivity (tasklist, client, xml,
+			   "never_group_radio",
+			   "auto_group_radio",
+			   "always_group_radio",
+			   "group_windows" /* key */);
+
 	tasklist->move_minimized_radio = WID ("move_minimized_radio");
 	tasklist->change_workspace_radio = WID ("change_workspace_radio");
 
+	setup_sensitivity (tasklist, client, xml,
+			   "move_minimized_radio",
+			   "change_workspace_radio",
+			   NULL,
+			   "move_unminimized_windows" /* key */);
+
 	tasklist->minimum_size_spin = WID ("minimum_size");
+	setup_sensitivity (tasklist, client, xml,
+			   "minimum_size",
+			   "minimum_size_label",
+			   "minimum_size_post_label",
+			   "minimum_size" /* key */);
+
 	tasklist->maximum_size_spin = WID ("maximum_size");
+	setup_sensitivity (tasklist, client, xml,
+			   "maximum_size",
+			   "maximum_size_label",
+			   "maximum_size_post_label",
+			   "maximum_size" /* key */);
 
 	/* Window grouping: */
 	button = get_grouping_button (tasklist, tasklist->grouping);
@@ -998,6 +1051,9 @@ display_properties_dialog (BonoboUIComponent *uic,
 		g_object_unref (G_OBJECT (xml));
 	}
 
+	gnome_window_icon_set_from_file (GTK_WINDOW (tasklist->properties_dialog), 
+					 GNOME_ICONDIR "/gnome-windows.png");
+	gtk_window_set_resizable (GTK_WINDOW (tasklist->properties_dialog), FALSE);
 	gtk_window_set_screen (GTK_WINDOW (tasklist->properties_dialog),
 			       gtk_widget_get_screen (tasklist->applet));
 	gtk_window_present (GTK_WINDOW (tasklist->properties_dialog));
