@@ -39,6 +39,8 @@
 #define MAX_REASONABLE_ROWS 16
 #define DEFAULT_ROWS 1
 
+#define NEVER_SENSITIVE "never_sensitive"
+
 typedef struct {
 	GtkWidget *applet;
 
@@ -55,6 +57,7 @@ typedef struct {
 	GtkWidget *label_row_col;
 	GtkWidget *num_workspaces_spin;
 	GtkWidget *workspaces_tree;
+	GtkWidget *about;
 
 	GtkListStore *workspaces_store;
 	
@@ -141,40 +144,33 @@ applet_change_orient (PanelApplet       *applet,
 	pager->orientation = new_orient;
 	pager_update (pager);
 	if (pager->label_row_col) 
-		gtk_label_set_text(GTK_LABEL(pager->label_row_col), pager->orientation == GTK_ORIENTATION_HORIZONTAL ? _("Rows") : _("Columns"));	
+		gtk_label_set_text (GTK_LABEL (pager->label_row_col), pager->orientation == GTK_ORIENTATION_HORIZONTAL ? _("rows") : _("columns"));	
 }
 
 static void 
-response_cb(GtkWidget * widget,int id, PagerData *pager)
+response_cb (GtkWidget *widget,
+	     int        id,
+	     PagerData *pager)
 {
-	if(id == GTK_RESPONSE_HELP) {
-		
+	if (id == GTK_RESPONSE_HELP) {
 		GError *error = NULL;
-		static GnomeProgram *applet_program = NULL;
-
-		if (!applet_program) {
-			int argc = 1;
-			char *argv[2] = { "workspace-switcher" };
-			applet_program = gnome_program_init ("workspace-switcher", VERSION,
-							     LIBGNOME_MODULE, argc, argv,
-							     GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
-		}
 
 		egg_help_display_desktop_on_screen (
-			applet_program, "workspace-switcher",
-			"workspace-switcher", "workspacelist-prefs",
+			NULL, "workspace-switcher", "workspace-switcher", "workspacelist-prefs",
 			gtk_widget_get_screen (pager->applet),
 			&error);
+
 		if (error) {
 			GtkWidget *dialog;
-			dialog = gtk_message_dialog_new (GTK_WINDOW(widget),
+
+			dialog = gtk_message_dialog_new (GTK_WINDOW (widget),
 							 GTK_DIALOG_DESTROY_WITH_PARENT,
 							 GTK_MESSAGE_ERROR,
-							 GTK_BUTTONS_CLOSE,
+							 GTK_BUTTONS_OK,
 							  _("There was an error displaying help: %s"),
 							 error->message);
 
-			g_signal_connect (G_OBJECT (dialog), "response",
+			g_signal_connect (dialog, "response",
 					  G_CALLBACK (gtk_widget_destroy),
 					  NULL);
 
@@ -183,11 +179,8 @@ response_cb(GtkWidget * widget,int id, PagerData *pager)
 					       gtk_widget_get_screen (pager->applet));
 			gtk_widget_show (dialog);
 			g_error_free (error);
-
 		}
-
-	}
-	else
+	} else
 		gtk_widget_hide (widget);
 }
 
@@ -207,6 +200,9 @@ destroy_pager(GtkWidget * widget, PagerData *pager)
 	if (pager->properties_dialog)
 		gtk_widget_destroy (pager->properties_dialog);
 
+	if (pager->about)
+		gtk_widget_destroy (pager->about);
+
 	g_free (pager);
 }
 
@@ -292,7 +288,8 @@ all_workspaces_changed (GConfClient *client,
 				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pager->current_only_radio), TRUE);
 			}
 		}
-		gtk_widget_set_sensitive (pager->num_rows_spin, value);
+		if ( ! g_object_get_data (G_OBJECT (pager->num_rows_spin), NEVER_SENSITIVE))
+			gtk_widget_set_sensitive (pager->num_rows_spin, value);
 	}
 }
 
@@ -445,31 +442,22 @@ display_help_dialog (BonoboUIComponent *uic,
 		     const gchar       *verbname)
 {
 	GError *error = NULL;
-	static GnomeProgram *applet_program = NULL;
-
-	if (!applet_program) {
-		int argc = 1;
-		char *argv[2] = { "workspace-switcher" };
-		applet_program = gnome_program_init ("workspace-switcher", VERSION,
-						     LIBGNOME_MODULE, argc, argv,
-						     GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
-	}
 
 	egg_help_display_desktop_on_screen (
-		applet_program, "workspace-switcher",
-		"workspace-switcher", NULL,
+		NULL, "workspace-switcher", "workspace-switcher", NULL,
 		gtk_widget_get_screen (pager->applet),
 		&error);
+
 	if (error) {
 		GtkWidget *dialog;
 		dialog = gtk_message_dialog_new (NULL,
 						 GTK_DIALOG_MODAL,
 						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
+						 GTK_BUTTONS_OK,
 						  _("There was an error displaying help: %s"),
 						 error->message);
 
-		g_signal_connect (G_OBJECT (dialog), "response",
+		g_signal_connect (dialog, "response",
 				  G_CALLBACK (gtk_widget_destroy),
 				  NULL);
 
@@ -487,7 +475,6 @@ display_about_dialog (BonoboUIComponent *uic,
 		      PagerData         *pager,
 		      const gchar       *verbname)
 {
-	static GtkWidget *about = NULL;
 	GdkPixbuf *pixbuf = NULL;
 	gchar *file;
 	
@@ -501,10 +488,10 @@ display_about_dialog (BonoboUIComponent *uic,
 	};
 	const char *translator_credits = _("translator_credits");
 
-	if (about) {
-		gtk_window_set_screen (GTK_WINDOW (about),
+	if (pager->about) {
+		gtk_window_set_screen (GTK_WINDOW (pager->about),
 				       gtk_widget_get_screen (pager->applet));
-		gtk_window_present (GTK_WINDOW (about));
+		gtk_window_present (GTK_WINDOW (pager->about));
 		return;
 	}
 
@@ -512,7 +499,7 @@ display_about_dialog (BonoboUIComponent *uic,
 	pixbuf = gdk_pixbuf_new_from_file (file, NULL);
 	g_free(file);
 
-	about = gnome_about_new (_("Workspace Switcher"), VERSION,
+	pager->about = gnome_about_new (_("Workspace Switcher"), VERSION,
 				 "Copyright \xc2\xa9 2001-2002 Red Hat, Inc.",
 				 _("The Workspace Switcher shows you a small version of your workspaces that lets you manage your windows."),
 				 authors,
@@ -520,19 +507,19 @@ display_about_dialog (BonoboUIComponent *uic,
 				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 				 pixbuf);
 	
-	gtk_window_set_wmclass (GTK_WINDOW (about), "pager", "Pager");
-	gtk_window_set_screen (GTK_WINDOW (about),
+	gtk_window_set_wmclass (GTK_WINDOW (pager->about), "pager", "Pager");
+	gtk_window_set_screen (GTK_WINDOW (pager->about),
 			       gtk_widget_get_screen (pager->applet));
 
 	if (pixbuf) {
-		gtk_window_set_icon (GTK_WINDOW (about), pixbuf);
+		gtk_window_set_icon (GTK_WINDOW (pager->about), pixbuf);
 		g_object_unref (pixbuf);
 	}
 	
-	g_signal_connect (G_OBJECT(about), "destroy",
-			  (GCallback)gtk_widget_destroyed, &about);
+	g_signal_connect (G_OBJECT(pager->about), "destroy",
+			  (GCallback)gtk_widget_destroyed, &pager->about);
 	
-	gtk_widget_show (about);
+	gtk_widget_show (pager->about);
 }
 
 
@@ -687,6 +674,50 @@ close_dialog (GtkWidget *button,
 	gtk_widget_hide (pager->properties_dialog);
 }
 
+static void
+setup_sensitivity (PagerData *pager,
+		   GladeXML *xml,
+		   const char *wid1,
+		   const char *wid2,
+		   const char *wid3,
+		   const char *key)
+{
+	PanelApplet *applet = PANEL_APPLET (pager->applet);
+	GConfClient *client = gconf_client_get_default ();
+	char *fullkey;
+	GtkWidget *w;
+
+	fullkey = panel_applet_gconf_get_full_key (applet, key);
+
+	if (gconf_client_key_is_writable (client, fullkey, NULL)) {
+		g_free (fullkey);
+		return;
+	}
+	g_free (fullkey);
+
+	w = glade_xml_get_widget (xml, wid1);
+	g_assert (w != NULL);
+	g_object_set_data (G_OBJECT (w), NEVER_SENSITIVE,
+			   GINT_TO_POINTER (1));
+	gtk_widget_set_sensitive (w, FALSE);
+
+	if (wid2 != NULL) {
+		w = glade_xml_get_widget (xml, wid2);
+		g_assert (w != NULL);
+		g_object_set_data (G_OBJECT (w), NEVER_SENSITIVE,
+				   GINT_TO_POINTER (1));
+		gtk_widget_set_sensitive (w, FALSE);
+	}
+	if (wid3 != NULL) {
+		w = glade_xml_get_widget (xml, wid3);
+		g_assert (w != NULL);
+		g_object_set_data (G_OBJECT (w), NEVER_SENSITIVE,
+				   GINT_TO_POINTER (1));
+		gtk_widget_set_sensitive (w, FALSE);
+	}
+
+}
+
 static void
 setup_dialog (GladeXML  *xml,
 	      PagerData *pager)
@@ -697,10 +728,28 @@ setup_dialog (GladeXML  *xml,
 	int nr_ws, i;
 	
 	pager->display_workspaces_toggle = WID ("workspace_name_toggle");
+	setup_sensitivity (pager, xml,
+			   "workspace_name_toggle",
+			   NULL,
+			   NULL,
+			   "display_workspace_names" /* key */);
+
 	pager->all_workspaces_radio = WID ("all_workspaces_radio");
 	pager->current_only_radio = WID ("current_only_radio");
+	setup_sensitivity (pager, xml,
+			   "all_workspaces_radio",
+			   "current_only_radio",
+			   "label_row_col",
+			   "display_all_workspaces" /* key */);
+
 	pager->num_rows_spin = WID ("num_rows_spin");
 	pager->label_row_col = WID("label_row_col");
+	setup_sensitivity (pager, xml,
+			   "num_rows_spin",
+			   NULL,
+			   NULL,
+			   "num_rows" /* key */);
+
 	pager->num_workspaces_spin = WID ("num_workspaces_spin");
 	pager->workspaces_tree = WID ("workspaces_tree_view");
 
@@ -723,7 +772,8 @@ setup_dialog (GladeXML  *xml,
 
 	if (pager->display_all) {
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pager->all_workspaces_radio), TRUE);
-		gtk_widget_set_sensitive (pager->num_rows_spin, TRUE);
+		if ( ! g_object_get_data (G_OBJECT (pager->num_rows_spin), NEVER_SENSITIVE))
+			gtk_widget_set_sensitive (pager->num_rows_spin, TRUE);
 	} else {
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pager->current_only_radio), TRUE);
 		gtk_widget_set_sensitive (pager->num_rows_spin, FALSE);
@@ -734,7 +784,7 @@ setup_dialog (GladeXML  *xml,
 			  (GCallback) num_rows_value_changed, pager);
 
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (pager->num_rows_spin), pager->n_rows);
-	gtk_label_set_text(GTK_LABEL(pager->label_row_col), pager->orientation == GTK_ORIENTATION_HORIZONTAL ? _("Rows") : _("Columns"));
+	gtk_label_set_text (GTK_LABEL (pager->label_row_col), pager->orientation == GTK_ORIENTATION_HORIZONTAL ? _("rows") : _("columns"));
 
 	g_signal_connect (pager->properties_dialog, "delete_event",
 			  G_CALLBACK (delete_event),
@@ -806,6 +856,8 @@ display_properties_dialog (BonoboUIComponent *uic,
 		g_object_unref (G_OBJECT (xml));
 	}
 
+	gnome_window_icon_set_from_file (GTK_WINDOW (pager->properties_dialog),
+	                                 GNOME_ICONDIR "/gnome-workspace.png");
 	gtk_window_set_screen (GTK_WINDOW (pager->properties_dialog),
 			       gtk_widget_get_screen (pager->applet));
 	gtk_window_present (GTK_WINDOW (pager->properties_dialog));
