@@ -11,22 +11,20 @@
 
 #include "button-widget.h"
 #include "panel-widget.h"
-#include "basep-widget.h"
-#include "panel-main.h"
 #include "panel-types.h"
 #include "panel-util.h"
 #include "panel-config-global.h"
 #include "panel-marshal.h"
 #include "panel-typebuiltins.h"
+#include "panel-globals.h"
+
 
 
 static GdkPixbuf *button_load_pixbuf (const char  *file,
-				      int          preffered_size,
 				      char       **error);
 
 enum {
 	PROP_0,
-	PROP_SIZE,
 	PROP_HAS_ARROW,
 	PROP_ORIENT,
 	PROP_ICON_NAME,
@@ -35,8 +33,6 @@ enum {
 
 #define BUTTON_WIDGET_DISPLACEMENT 2
 
-extern GlobalConfig global_config;
-
 static GObjectClass *parent_class;
 
 static void
@@ -51,16 +47,6 @@ translate_to(GtkWidget *from, GtkWidget *to, int *x, int *y)
 	}
 }
 
-static GtkWidget *
-get_frame(BasePWidget *basep)
-{
-	if (GTK_WIDGET_VISIBLE (basep->frame)) {
-		return basep->frame;
-	} else {
-		return basep->innerebox;
-	}
-}
-
 static void
 calculate_overlay_geometry (PanelWidget *panel,
 			    GtkWidget   *parent,
@@ -94,23 +80,6 @@ calculate_overlay_geometry (PanelWidget *panel,
 		if ((*w + applet->allocation.x) > panel->size) {
 			*w = panel->size - applet->allocation.x;
 		}
-
-		if ( ! BASEP_IS_WIDGET(parent)) {
-			/*don't do the edge flushing on foobar*/
-			return;
-		}
-
-		/* if on the edge (only if padding is 0)
-		   then make the thing flush with the innerebox or frame
-		   of the basep */
-		if(applet->allocation.x == 0) {
-			GtkWidget *frame = get_frame(BASEP_WIDGET(parent));
-			*w += (*x - frame->allocation.x);
-			*x = frame->allocation.x;
-		} else if(applet->allocation.x + *w == panel->size) {
-			GtkWidget *frame = get_frame(BASEP_WIDGET(parent));
-			*w = frame->allocation.width + frame->allocation.x - *x;
-		}
 	} else {
 		if (applet->allocation.y > panel->size) {
 			*x = parent->requisition.width + 1;
@@ -125,23 +94,6 @@ calculate_overlay_geometry (PanelWidget *panel,
 		if ((*h + applet->allocation.y) > panel->size) {
 			*h = panel->size - applet->allocation.y;
 		}
-
-		if ( ! BASEP_IS_WIDGET(parent)) {
-			/*don't do the edge flushing on foobar*/
-			return;
-		}
-
-		/* if on the edge (only if padding is 0)
-		   then make the thing flush with the innerbox of frame
-		   of the basep */
-		if(applet->allocation.y == 0) {
-			GtkWidget *frame = get_frame(BASEP_WIDGET(parent));
-			*h += (*y - frame->allocation.y);
-			*y = frame->allocation.y;
-		} else if(applet->allocation.y + *h == panel->size) {
-			GtkWidget *frame = get_frame(BASEP_WIDGET(parent));
-			*h = frame->allocation.height + frame->allocation.y - *y;
-		}
 	}
 }
 
@@ -216,8 +168,8 @@ button_widget_realize(GtkWidget *widget)
 	g_return_if_fail (widget != NULL);
 	g_return_if_fail (BUTTON_IS_WIDGET (widget));
 
-	panel = PANEL_WIDGET(widget->parent);
-	parent = panel->panel_parent;
+	panel  = PANEL_WIDGET(widget->parent);
+	parent = GTK_WIDGET (panel->toplevel);
 
 	calculate_overlay_geometry(panel, parent, widget, &x, &y, &w, &h);
 
@@ -263,7 +215,7 @@ button_widget_parent_set (GtkWidget *widget,
 	if (!GTK_WIDGET_REALIZED (widget)|| !widget->parent)
 		return;
 
-	parent = PANEL_WIDGET (widget->parent)->panel_parent;
+	parent = GTK_WIDGET (PANEL_WIDGET (widget->parent)->toplevel);
 
 	calculate_overlay_geometry (
 		PANEL_WIDGET (widget->parent), parent, widget, &x, &y, &w, &h);
@@ -289,13 +241,9 @@ button_widget_unset_pixbufs (ButtonWidget *button)
 }
 
 static void
-button_widget_load_pixbuf_and_scale (ButtonWidget *button,
-				     int           dest_width,
-				     int           dest_height)
+button_widget_load_pixbuf_and_scale (ButtonWidget *button)
 {
 	double scale;
-	double scale_x;
-	double scale_y;
 	int    width;
 	int    height;
 
@@ -314,8 +262,7 @@ button_widget_load_pixbuf_and_scale (ButtonWidget *button,
 		else {
 			char *error = NULL;
 
-			button->pixbuf = button_load_pixbuf (
-						button->filename, button->size, &error);
+			button->pixbuf = button_load_pixbuf (button->filename, &error);
 			if (error) {
 				panel_error_dialog (gdk_screen_get_default (),
 						    "cannot_load_pixbuf",
@@ -331,13 +278,16 @@ button_widget_load_pixbuf_and_scale (ButtonWidget *button,
 			return;
 	}
 
+	if (button->size <= 1)
+		return;
+
 	width  = gdk_pixbuf_get_width  (button->pixbuf);
 	height = gdk_pixbuf_get_height (button->pixbuf);
 
-	scale_x = (double) dest_width  / width;
-	scale_y = (double) dest_height / height;
-
-	scale = MIN (scale_x, scale_y);
+	if (button->orientation & PANEL_HORIZONTAL_MASK)
+		scale = (double) button->size / height;
+	else
+		scale = (double) button->size / width;
 
 	width  *= scale;
 	height *= scale;
@@ -357,21 +307,15 @@ button_widget_load_pixbuf_and_scale (ButtonWidget *button,
 		g_object_unref (button->scaled_hc);
 	
 	button->scaled_hc = make_hc_pixbuf (button->scaled);
+
+	gtk_widget_queue_resize (GTK_WIDGET (button));
 }
 
 static void
 button_widget_reload_pixbuf (ButtonWidget *button)
 {
 	button_widget_unset_pixbufs (button);
-	if (GTK_WIDGET (button)->allocation.width  <= 1 ||
-	    GTK_WIDGET (button)->allocation.height <= 1)
-		return;
-
-	button_widget_load_pixbuf_and_scale (
-		button,
-		GTK_WIDGET (button)->allocation.width,
-		GTK_WIDGET (button)->allocation.height);
-	gtk_widget_queue_draw (GTK_WIDGET (button));
+	button_widget_load_pixbuf_and_scale (button);
 }
 
 static void
@@ -413,14 +357,11 @@ button_widget_get_property (GObject    *object,
 	button = BUTTON_WIDGET (object);
 
 	switch (prop_id) {
-	case PROP_SIZE:
-		g_value_set_int (value, button->size);
-		break;
 	case PROP_HAS_ARROW:
 		g_value_set_boolean (value, button->arrow);
 		break;
 	case PROP_ORIENT:
-		g_value_set_enum (value, button->orient);
+		g_value_set_enum (value, button->orientation);
 		break;
 	case PROP_ICON_NAME:
 		g_value_set_string (value, button->filename);
@@ -450,16 +391,12 @@ button_widget_set_property (GObject      *object,
 		const char *icon_name;
 		const char *stock_id;
 
-	case PROP_SIZE:
-		button->size = g_value_get_int (value);
-		gtk_widget_queue_resize (GTK_WIDGET (button));
-		break;
 	case PROP_HAS_ARROW:
 		button->arrow = g_value_get_boolean (value) ? 1 : 0;
 		gtk_widget_queue_draw (GTK_WIDGET (button));
 		break;
 	case PROP_ORIENT:
-		button->orient = g_value_get_enum (value);
+		button->orientation = g_value_get_enum (value);
 		gtk_widget_queue_draw (GTK_WIDGET (button));
 		break;
 	case PROP_ICON_NAME:
@@ -549,20 +486,19 @@ load_pixbuf (const char  *file,
 
 static GdkPixbuf *
 button_load_pixbuf (const char  *file,
-		    int          preffered_size,
 		    char       **error)
 {
+/* FIXME: this should be based on the panel size */
+#define PREFERRED_SIZE 48
+
 	GdkPixbuf *retval = NULL;
 	char *tmp;
 
-	if (preffered_size <= 0)
-		preffered_size = 48;
-	
 	if (string_empty (file))
-		return get_missing (preffered_size);
+		return get_missing (PREFERRED_SIZE);
 
 	tmp = g_path_get_basename (file);
-	retval = load_pixbuf (tmp, preffered_size, error);
+	retval = load_pixbuf (tmp, PREFERRED_SIZE, error);
 	g_free (tmp);
 
 	if (!retval && g_path_is_absolute (file)) {
@@ -571,52 +507,59 @@ button_load_pixbuf (const char  *file,
 			*error = NULL;
 		}
 
-		retval = load_pixbuf (file, preffered_size, error);
+		retval = load_pixbuf (file, PREFERRED_SIZE, error);
 	}
 
 	if (!retval)
-		retval = get_missing (preffered_size);
+		retval = get_missing (PREFERRED_SIZE);
 
 	return retval;
+
+#undef PREFERRED_SIZE
 }
 
-#define SCALE(x) (((x)*size)/48.0)
-
 static void
-draw_arrow(GdkPoint *points, PanelOrient orient, int size)
+draw_arrow (GdkPoint         *points,
+	    PanelOrientation  orientation,
+	    int               width,
+	    int               height)
 {
-	switch(orient) {
-	case PANEL_ORIENT_UP:
-		points[0].x = SCALE(48-12);
-		points[0].y = SCALE(10);
-		points[1].x = SCALE(48-4);
-		points[1].y = SCALE(10);
-		points[2].x = SCALE(48-8);
-		points[2].y = SCALE(3);
+	double scale;
+
+	scale = (orientation & PANEL_HORIZONTAL_MASK ? height : width) / 48.0;
+
+	switch (orientation) {
+	case PANEL_ORIENTATION_TOP:
+		points [0].x = 4 * scale;
+		points [0].y = height - 10 * scale;
+		points [1].x = 12 * scale;
+		points [1].y = height - 10 * scale;
+		points [2].x = 8 * scale;
+		points [2].y = height - 3 * scale;
 		break;
-	case PANEL_ORIENT_DOWN:
-		points[0].x = SCALE(4);
-		points[0].y = SCALE(48 - 10);
-		points[1].x = SCALE(12);
-		points[1].y = SCALE(48 - 10);
-		points[2].x = SCALE(8);
-		points[2].y = SCALE(48 - 3);
+	case PANEL_ORIENTATION_BOTTOM:
+		points [0].x = width - 12 * scale;
+		points [0].y = 10 * scale;
+		points [1].x = width - 4 * scale;
+		points [1].y = 10 * scale;
+		points [2].x = width - 8 * scale;
+		points [2].y = 3 * scale;
 		break;
-	case PANEL_ORIENT_LEFT:
-		points[0].x = SCALE(10);
-		points[0].y = SCALE(4);
-		points[1].x = SCALE(10);
-		points[1].y = SCALE(12);
-		points[2].x = SCALE(3);
-		points[2].y = SCALE(8);
+	case PANEL_ORIENTATION_LEFT:
+		points [0].x = width  - 10 * scale;
+		points [0].y = height - 12 * scale;
+		points [1].x = width  - 10 * scale;
+		points [1].y = height - 4 * scale;
+		points [2].x = width  - 3 * scale;
+		points [2].y = height - 8 * scale;
 		break;
-	case PANEL_ORIENT_RIGHT:
-		points[0].x = SCALE(48 - 10);
-		points[0].y = SCALE(48 - 12);
-		points[1].x = SCALE(48 - 10);
-		points[1].y = SCALE(48 - 4);
-		points[2].x = SCALE(48 - 3);
-		points[2].y = SCALE(48 - 8);
+	case PANEL_ORIENTATION_RIGHT:
+		points [0].x = 10 * scale;
+		points [0].y = 4  * scale;
+		points [1].x = 10 * scale;
+		points [1].y = 12 * scale;
+		points [2].x = 3  * scale;
+		points [2].y = 8  * scale;
 		break;
 	}
 }
@@ -640,7 +583,7 @@ button_widget_expose (GtkWidget         *widget,
 	ButtonWidget *button_widget;
 	GtkButton *button;
 	GdkRectangle area, image_bound;
-	int off, size;
+	int off;
 	int x, y, w, h;
 	GdkPixbuf *pb = NULL;
   
@@ -654,16 +597,18 @@ button_widget_expose (GtkWidget         *widget,
 		return FALSE;
 	}
 
-	size = widget->allocation.height;
 	/* offset for pressed buttons */
 	off = (button->in_button && button->button_down) ?
-		SCALE(BUTTON_WIDGET_DISPLACEMENT) : 0;
+		BUTTON_WIDGET_DISPLACEMENT * widget->allocation.height / 48.0 : 0;
 	
-	if (global_config.highlight_when_over && 
+	if (panel_global_config_get_highlight_when_over () && 
 	    (button->in_button || GTK_WIDGET_HAS_FOCUS (widget)))
 		pb = button_widget->scaled_hc;
 	else
 		pb = button_widget->scaled;
+
+	if (!pb)
+		return FALSE;
 	
 	w = gdk_pixbuf_get_width (pb);
 	h = gdk_pixbuf_get_height (pb);
@@ -678,22 +623,18 @@ button_widget_expose (GtkWidget         *widget,
 	area = event->area;
 	
 	if (gdk_rectangle_intersect (&area, &widget->allocation, &area) &&
-	    gdk_rectangle_intersect (&image_bound, &area, &image_bound))  {
-		gdk_pixbuf_render_to_drawable_alpha (pb,
-						     widget->window,
-						     image_bound.x - x, image_bound.y - y,
-						     image_bound.x, image_bound.y,
-						     image_bound.width, image_bound.height,
-						     GDK_PIXBUF_ALPHA_FULL,
-						     128,
-						     GDK_RGB_DITHER_NORMAL,
-						     0, 0);
-	}
+	    gdk_rectangle_intersect (&image_bound, &area, &image_bound))
+		gdk_draw_pixbuf (widget->window, NULL, pb,
+				 image_bound.x - x, image_bound.y - y,
+				 image_bound.x, image_bound.y,
+				 image_bound.width, image_bound.height,
+				 GDK_RGB_DITHER_NORMAL,
+				 0, 0);
 	
-	if(button_widget->arrow) {
+	if (button_widget->arrow) {
 		int i;
 		GdkPoint points[3];
-		draw_arrow (points, button_widget->orient, widget->allocation.height);
+		draw_arrow (points, button_widget->orientation, widget->allocation.width, widget->allocation.height);
 		for (i = 0; i < 3; i++) {
 			points[i].x += off + widget->allocation.x;
 			points[i].y += off + widget->allocation.y;
@@ -734,36 +675,44 @@ static void
 button_widget_size_request (GtkWidget      *widget,
 			    GtkRequisition *requisition)
 {
-	PanelWidget *panel = PANEL_WIDGET (widget->parent);
+	ButtonWidget *button_widget = BUTTON_WIDGET (widget);
 
-	requisition->width = requisition->height = panel->sz;
+	if (button_widget->scaled) {
+		requisition->width  = gdk_pixbuf_get_width  (button_widget->scaled);
+		requisition->height = gdk_pixbuf_get_height (button_widget->scaled);
+	}
 }
 
 static void
 button_widget_size_allocate (GtkWidget     *widget,
 			     GtkAllocation *allocation)
 {
-	ButtonWidget *button_widget;
-	GtkButton    *button;
+	ButtonWidget *button_widget = BUTTON_WIDGET (widget);
+	GtkButton    *button = GTK_BUTTON (widget);
+	int           size;
 
-	g_return_if_fail (BUTTON_IS_WIDGET (widget));
+	if (button_widget->orientation & PANEL_HORIZONTAL_MASK)
+		size = allocation->height;
+	else
+		size = allocation->width;
 
-	button_widget = BUTTON_WIDGET (widget);
-	button        = GTK_BUTTON (widget);
+	if (button_widget->size != size) {
+		button_widget->size = size;
+
+		button_widget_load_pixbuf_and_scale (button_widget);
+	}
 
 	widget->allocation = *allocation;
 
-	button_widget_load_pixbuf_and_scale (
-		button_widget, allocation->width, allocation->height);
-
 	if (GTK_WIDGET_REALIZED (widget)) {
 		PanelWidget *panel;
 		int          x, y, w, h;
 
 		panel = PANEL_WIDGET (widget->parent);
 
-		calculate_overlay_geometry (panel, panel->panel_parent,
-					    widget, &x, &y, &w, &h);
+		calculate_overlay_geometry (
+			panel, GTK_WIDGET (panel->toplevel),
+			widget, &x, &y, &w, &h);
 
 		gdk_window_move_resize (button->event_window, x, y, w, h);
 	}
@@ -821,7 +770,7 @@ button_widget_leave_notify (GtkWidget *widget, GdkEventCrossing *event)
 	in_button = GTK_BUTTON (widget)->in_button;
 	GTK_WIDGET_CLASS (parent_class)->leave_notify_event (widget, event);
 	if (in_button != GTK_BUTTON (widget)->in_button &&
-	    global_config.highlight_when_over)
+	    panel_global_config_get_highlight_when_over ())
 		gtk_widget_queue_draw (widget);
 
 	return FALSE;
@@ -858,14 +807,14 @@ button_widget_instance_init (ButtonWidget *button)
 	button->scaled    = NULL;
 	button->scaled_hc = NULL;
 	
-	button->arrow  = 0;
-	button->size   = -1;
-	button->orient = PANEL_ORIENT_UP;
+	button->arrow       = 0;
+	button->orientation = PANEL_ORIENTATION_TOP;
 	
 	button->ignore_leave  = FALSE;
 	button->dnd_highlight = FALSE;
 
 	button->pressed_timeout = 0;
+	button->size            = 0;
 
 	panel_signal_connect_object_while_alive (
 			panel_icon_theme, "changed",
@@ -897,15 +846,6 @@ button_widget_class_init (ButtonWidgetClass *klass)
 	button_class->pressed  = button_widget_button_pressed;
 	button_class->released = button_widget_button_released;
 
-	g_object_class_install_property (
-			gobject_class,
-			PROP_SIZE,
-			g_param_spec_int ("size",
-					  _("Size"),
-					  _("The desired ButtonWidget size"),
-					  G_MININT, G_MAXINT, -1,
-					  G_PARAM_READWRITE));
-
 	g_object_class_install_property (
 			gobject_class,
 			PROP_HAS_ARROW,
@@ -921,8 +861,8 @@ button_widget_class_init (ButtonWidgetClass *klass)
 			g_param_spec_enum ("orient",
 					   _("Orientation"),
 					   _("The ButtonWidget orientation"),
-					   PANEL_TYPE_ORIENT,
-					   PANEL_ORIENT_UP,
+					   PANEL_TYPE_ORIENTATION,
+					   PANEL_ORIENTATION_TOP,
 					   G_PARAM_READWRITE));
 
 	g_object_class_install_property (
@@ -970,18 +910,16 @@ button_widget_get_type (void)
 }
 
 GtkWidget *
-button_widget_new (const char  *filename,
-		   int          size,
-		   gboolean     arrow,
-		   PanelOrient  orient)
+button_widget_new (const char       *filename,
+		   gboolean          arrow,
+		   PanelOrientation  orientation)
 {
 	GtkWidget *retval;
 
 	retval = g_object_new (
 			BUTTON_TYPE_WIDGET,
-			"size", size,
 			"has-arrow", arrow,
-			"orient", orient,
+			"orient", orientation,
 			"icon-name", filename,
 			NULL);
 	
@@ -989,18 +927,16 @@ button_widget_new (const char  *filename,
 }
 
 GtkWidget *
-button_widget_new_from_stock (const char  *stock_id,
-			      int          size,
-			      gboolean     arrow,
-			      PanelOrient  orient)
+button_widget_new_from_stock (const char       *stock_id,
+			      gboolean          arrow,
+			      PanelOrientation  orientation)
 {
 	GtkWidget *retval;
 
 	retval = g_object_new (
 			BUTTON_TYPE_WIDGET,
-			"size", size,
 			"has-arrow", arrow,
-			"orient", orient,
+			"orient", orientation,
 			"stock-id", stock_id,
 			NULL);
 	
@@ -1026,12 +962,12 @@ button_widget_set_stock_id (ButtonWidget *button,
 }
 
 void
-button_widget_set_params(ButtonWidget *button,
-			 gboolean arrow,
-			 PanelOrient orient)
+button_widget_set_params (ButtonWidget     *button,
+			  gboolean          arrow,
+			  PanelOrientation  orientation)
 {
 	g_return_if_fail (BUTTON_IS_WIDGET (button));
 
 	g_object_set (G_OBJECT (button), "has-arrow", arrow, NULL);
-	g_object_set (G_OBJECT (button), "orient", orient, NULL);
+	g_object_set (G_OBJECT (button), "orient", orientation, NULL);
 }
