@@ -56,8 +56,8 @@ struct _PanelMenuBarPrivate {
 static GObjectClass *parent_class;
 
 static void
-panel_menu_bar_show_applications_menu (GtkWidget    *menu,
-				       PanelMenuBar *menubar)
+panel_menu_bar_show_applications_menu (PanelMenuBar *menubar,
+				       GtkWidget    *menu)
 {
 	gtk_menu_set_screen (GTK_MENU (menu),
 			     gtk_widget_get_screen (GTK_WIDGET (menubar)));
@@ -107,7 +107,7 @@ panel_menu_bar_append_actions_menu (PanelMenuBar *menubar)
 	panel_menu_bar_append_action_item (
 			menubar,
 			menubar->priv->actions_menu,
-			_("Run Application"),
+			_("Run Application..."),
 			PANEL_STOCK_RUN,
 			_("Run an Application by entering a command"),
 			"ACTION:run:NEW",
@@ -135,7 +135,7 @@ panel_menu_bar_append_actions_menu (PanelMenuBar *menubar)
 		panel_menu_bar_append_action_item (
 			menubar,
 			menubar->priv->actions_menu,
-			_("Screenshot..."),
+			_("Take Screenshot..."),
 			PANEL_STOCK_SCREENSHOT,
 			_("Take a screenshot of your desktop"),
 			"ACTION:screenshot:NEW",
@@ -197,9 +197,11 @@ panel_menu_bar_instance_init (PanelMenuBar      *menubar,
 				   menubar->priv->applications_menu);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menubar), item);
 
-	g_signal_connect (menubar->priv->applications_menu, "show",
-			  G_CALLBACK (panel_menu_bar_show_applications_menu), menubar);
-
+	g_signal_connect_swapped (menubar->priv->applications_menu, "show",
+				  G_CALLBACK (panel_menu_bar_show_applications_menu), menubar);
+	g_signal_connect_swapped (menubar->priv->applications_menu, "hide",
+				  G_CALLBACK (gtk_menu_shell_deselect), menubar);
+	
 	panel_menu_bar_append_actions_menu (menubar);
 }
 
@@ -285,6 +287,7 @@ panel_menu_bar_get_type (void)
 
 static void
 panel_menu_bar_load (PanelWidget *panel,
+		     gboolean     locked,
 		     int          position,
 		     gboolean     exactpos,
 		     const char  *id)
@@ -296,8 +299,9 @@ panel_menu_bar_load (PanelWidget *panel,
 	menubar = g_object_new (PANEL_TYPE_MENU_BAR, NULL);
 
 	menubar->priv->info = panel_applet_register (
-					GTK_WIDGET (menubar), NULL, NULL, panel,
-					position, exactpos, PANEL_OBJECT_MENU_BAR, id);
+					GTK_WIDGET (menubar), NULL, NULL,
+					panel, locked, position, exactpos,
+					PANEL_OBJECT_MENU_BAR, id);
 	if (!menubar->priv->info) {
 		gtk_widget_destroy (GTK_WIDGET (menubar));
 		return;
@@ -311,11 +315,12 @@ panel_menu_bar_load (PanelWidget *panel,
 
 void
 panel_menu_bar_load_from_gconf (PanelWidget *panel,
+				gboolean     locked,
 				int          position,
 				gboolean     exactpos,
 				const char  *id)
 {
-	panel_menu_bar_load (panel, position, exactpos, id);
+	panel_menu_bar_load (panel, locked, position, exactpos, id);
 }
 
 void
@@ -324,7 +329,7 @@ panel_menu_bar_create (PanelToplevel *toplevel,
 {
 	char *id;
 
-	id = panel_profile_prepare_object (PANEL_OBJECT_MENU_BAR, toplevel, position);
+	id = panel_profile_prepare_object (PANEL_OBJECT_MENU_BAR, toplevel, position, FALSE);
 
 	/* frees id */
 	panel_profile_add_to_list (PANEL_GCONF_OBJECTS, id);
@@ -346,3 +351,17 @@ panel_menu_bar_invoke_menu (PanelMenuBar *menubar,
 
 	panel_show_help (screen, "wgospanel.xml", NULL);
 }
+
+void
+panel_menu_bar_popup_menu (PanelMenuBar *menubar,
+			   guint32       activate_time)
+{
+	GtkMenu *menu;
+	
+	g_return_if_fail (PANEL_IS_MENU_BAR (menubar));
+
+	menu = GTK_MENU (menubar->priv->applications_menu);
+
+	gtk_menu_shell_select_item (GTK_MENU_SHELL (menubar),
+				    gtk_menu_get_attach_widget (menu));
+}
