@@ -27,10 +27,8 @@
 #include <libgnomeui/libgnomeui.h>
 #include <libgnome/libgnome.h>
 
-#include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "panel-util.h"
 
