@@ -1329,16 +1329,8 @@ panel_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 		GtkRequisition chreq;
 		gtk_widget_size_request(ad->applet,&chreq);
 
-		if (ad->expand_major && PANEL_IS_APPLET_FRAME (ad->applet)) {
-			g_free (ad->size_hints);
-
-			ad->size_hints_len =
-				panel_applet_frame_get_size_hints (
-					PANEL_APPLET_FRAME (ad->applet), &ad->size_hints);
-		}
-		
 		if (panel->orient == GTK_ORIENTATION_HORIZONTAL) {
-			if (requisition->height < chreq.height)
+			if (requisition->height < chreq.height && !ad->size_constrained)
 				requisition->height = chreq.height;
 
 			if (panel->packed && ad->expand_major && ad->size_hints)
@@ -1347,7 +1339,7 @@ panel_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 			else if (panel->packed)
 				requisition->width += chreq.width;
 		} else {
-			if (requisition->width < chreq.width)
+			if (requisition->width < chreq.width && !ad->size_constrained)
 				requisition->width = chreq.width;
 
 			if (panel->packed && ad->expand_major && ad->size_hints)
@@ -1749,6 +1741,7 @@ panel_widget_applet_drag_start_no_grab (PanelWidget *panel,
 					int drag_off)
 {
 	AppletData *ad;
+	AppletInfo *info;
 
 	g_return_val_if_fail (PANEL_IS_WIDGET (panel), FALSE);
 	g_return_val_if_fail (GTK_IS_WIDGET (panel), FALSE);
@@ -1759,6 +1752,13 @@ panel_widget_applet_drag_start_no_grab (PanelWidget *panel,
 	if (ad->locked)
 		return FALSE;
 
+	/* Check if we can actually move this object in the
+	   configuration */
+	info = g_object_get_data (G_OBJECT (applet), "applet_info");
+	if (info != NULL &&
+	    ! panel_applet_can_freely_move (info))
+		return FALSE;
+
 	if (moving_timeout != 0) {
 		g_source_remove (moving_timeout);
 		moving_timeout = 0;
@@ -2462,6 +2462,7 @@ panel_widget_add (PanelWidget *panel,
 		ad->drag_off = 0;
 		ad->dirty = FALSE;
 		ad->no_die = 0;
+		ad->size_constrained = FALSE;
 		ad->expand_major = FALSE;
 		ad->expand_minor = FALSE;
 		ad->locked = locked;
@@ -2768,6 +2769,27 @@ panel_widget_get_applet_orientation (PanelWidget *panel)
 	return panel_toplevel_get_orientation (panel->toplevel);
 }
 
+void
+panel_widget_set_applet_size_constrained (PanelWidget *panel,
+					  GtkWidget   *applet,
+					  gboolean     size_constrained)
+{
+	AppletData *ad;
+
+	ad = g_object_get_data (G_OBJECT (applet), PANEL_APPLET_DATA);
+	if (!ad)
+		return;
+
+	size_constrained = size_constrained != FALSE;
+
+	if (ad->size_constrained == size_constrained)
+		return;
+
+	ad->size_constrained = size_constrained;
+
+	gtk_widget_queue_resize (GTK_WIDGET (panel));
+}
+
 void
 panel_widget_set_applet_expandable (PanelWidget *panel,
 				    GtkWidget   *applet,
@@ -2780,6 +2802,9 @@ panel_widget_set_applet_expandable (PanelWidget *panel,
 	if (!ad)
 		return;
 
+	major = major != FALSE;
+	minor = minor != FALSE;
+
 	if (ad->expand_major == major && ad->expand_minor == minor)
 		return;
 
@@ -2789,6 +2814,26 @@ panel_widget_set_applet_expandable (PanelWidget *panel,
 	gtk_widget_queue_resize (GTK_WIDGET (panel));
 }
 
+void
+panel_widget_set_applet_size_hints (PanelWidget *panel,
+				    GtkWidget   *applet,
+				    int         *size_hints,
+				    int          size_hints_len)
+{
+	AppletData *ad;
+
+	ad = g_object_get_data (G_OBJECT (applet), PANEL_APPLET_DATA);
+	if (!ad)
+		return;
+
+	g_free (ad->size_hints);
+
+	ad->size_hints     = size_hints;
+	ad->size_hints_len = size_hints_len;
+
+	gtk_widget_queue_resize (GTK_WIDGET (panel));
+}
+
 void
 panel_widget_set_applet_locked (PanelWidget *panel,
 				GtkWidget   *applet,
